# Generated by ariadne-codegen
# Source: combined.graphql

from .base_model import BaseModel
from .fragments import ModuleInstanceMinimal


class ModuleInstanceStatus(BaseModel):
    module_instance: "ModuleInstanceStatusModuleInstance"


class ModuleInstanceStatusModuleInstance(ModuleInstanceMinimal):
    pass


ModuleInstanceStatus.model_rebuild()
ModuleInstanceStatusModuleInstance.model_rebuild()
