# Generated by ariadne-codegen
# Source: combined.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import ModuleFull, PageInfoFull


class Modules(BaseModel):
    modules: "ModulesModules"


class ModulesModules(BaseModel):
    page_info: "ModulesModulesPageInfo" = Field(alias="pageInfo")
    edges: List["ModulesModulesEdges"]


class ModulesModulesPageInfo(PageInfoFull):
    pass


class ModulesModulesEdges(BaseModel):
    cursor: str
    node: "ModulesModulesEdgesNode"


class ModulesModulesEdgesNode(ModuleFull):
    pass


Modules.model_rebuild()
ModulesModules.model_rebuild()
ModulesModulesPageInfo.model_rebuild()
ModulesModulesEdges.model_rebuild()
ModulesModulesEdgesNode.model_rebuild()
