# Generated by ariadne-codegen
# Source: combined.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import ModuleFull, PageInfoFull


class LatestModules(BaseModel):
    latest_modules: "LatestModulesLatestModules"


class LatestModulesLatestModules(BaseModel):
    page_info: "LatestModulesLatestModulesPageInfo" = Field(alias="pageInfo")
    edges: List["LatestModulesLatestModulesEdges"]


class LatestModulesLatestModulesPageInfo(PageInfoFull):
    pass


class LatestModulesLatestModulesEdges(BaseModel):
    cursor: str
    node: "LatestModulesLatestModulesEdgesNode"


class LatestModulesLatestModulesEdgesNode(ModuleFull):
    pass


LatestModules.model_rebuild()
LatestModulesLatestModules.model_rebuild()
LatestModulesLatestModulesPageInfo.model_rebuild()
LatestModulesLatestModulesEdges.model_rebuild()
LatestModulesLatestModulesEdgesNode.model_rebuild()
