# Generated by ariadne-codegen
# Source: combined.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import ArgumentFull, PageInfoFull


class Arguments(BaseModel):
    me: "ArgumentsMe"


class ArgumentsMe(BaseModel):
    account: "ArgumentsMeAccount"


class ArgumentsMeAccount(BaseModel):
    arguments: "ArgumentsMeAccountArguments"


class ArgumentsMeAccountArguments(BaseModel):
    page_info: "ArgumentsMeAccountArgumentsPageInfo" = Field(alias="pageInfo")
    edges: List["ArgumentsMeAccountArgumentsEdges"]


class ArgumentsMeAccountArgumentsPageInfo(PageInfoFull):
    pass


class ArgumentsMeAccountArgumentsEdges(BaseModel):
    cursor: str
    node: "ArgumentsMeAccountArgumentsEdgesNode"


class ArgumentsMeAccountArgumentsEdgesNode(ArgumentFull):
    pass


Arguments.model_rebuild()
ArgumentsMe.model_rebuild()
ArgumentsMeAccount.model_rebuild()
ArgumentsMeAccountArguments.model_rebuild()
ArgumentsMeAccountArgumentsPageInfo.model_rebuild()
ArgumentsMeAccountArgumentsEdges.model_rebuild()
ArgumentsMeAccountArgumentsEdgesNode.model_rebuild()
