# Generated by ariadne-codegen
# Source: combined.graphql

from .base_model import BaseModel
from .fragments import ArgumentFull


class Argument(BaseModel):
    argument: "ArgumentArgument"


class ArgumentArgument(ArgumentFull):
    pass


Argument.model_rebuild()
ArgumentArgument.model_rebuild()
