# Generated by ariadne-codegen
# Source: combined.graphql

from .base_model import BaseModel
from .fragments import ModuleInstanceCommon


class ModuleInstanceMinimal(BaseModel):
    module_instance: "ModuleInstanceMinimalModuleInstance"


class ModuleInstanceMinimalModuleInstance(ModuleInstanceCommon):
    pass


ModuleInstanceMinimal.model_rebuild()
ModuleInstanceMinimalModuleInstance.model_rebuild()
