# Generated by ariadne-codegen
# Source: schema.graphql

from enum import Enum


class MemUnits(str, Enum):
    B = "B"
    KB = "KB"
    MB = "MB"
    GB = "GB"
    TB = "TB"


class ModuleFailureReason(str, Enum):
    RESOLUTION = "RESOLUTION"
    DISPATCH = "DISPATCH"
    RUN = "RUN"
    STALLED = "STALLED"
    MISSING = "MISSING"
    CORRUPTED = "CORRUPTED"


class ModuleInstanceStatus(str, Enum):
    CREATED = "CREATED"
    RESOLVING = "RESOLVING"
    ADMITTED = "ADMITTED"
    DISPATCHED = "DISPATCHED"
    QUEUED = "QUEUED"
    RUNNING = "RUNNING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"


class ModuleInstanceTarget(str, Enum):
    NIX = "NIX"
    GADI = "GADI"
    NIX_SSH = "NIX_SSH"
    NIX_SSH_2 = "NIX_SSH_2"
    NIX_SSH_3 = "NIX_SSH_3"
    NIX_SSH_4 = "NIX_SSH_4"
    SETONIX = "SETONIX"
    GADI_GPU = "GADI_GPU"
    NIX_SSH_GPU = "NIX_SSH_GPU"
    NIX_SSH_2_GPU = "NIX_SSH_2_GPU"
    NIX_SSH_3_GPU = "NIX_SSH_3_GPU"
    NIX_SSH_4_GPU = "NIX_SSH_4_GPU"
    SETONIX_GPU = "SETONIX_GPU"
    GADI_EXPRESS = "GADI_EXPRESS"
    SETONIX_EXPRESS = "SETONIX_EXPRESS"


class OrderBy(str, Enum):
    ASC = "ASC"
    DESC = "DESC"
