# Generated by ariadne-codegen
# Source: combined.graphql

from typing import Any

from .base_model import BaseModel


class TrackUtilization(BaseModel):
    track_module_instance_resource_utilization: "TrackUtilizationTrackModuleInstanceResourceUtilization"


class TrackUtilizationTrackModuleInstanceResourceUtilization(BaseModel):
    id: Any


TrackUtilization.model_rebuild()
TrackUtilizationTrackModuleInstanceResourceUtilization.model_rebuild()
