# Generated by ariadne-codegen
# Source: combined.graphql

from uuid import UUID

from .base_model import BaseModel


class DeleteModuleInstance(BaseModel):
    delete_module_instance: "DeleteModuleInstanceDeleteModuleInstance"


class DeleteModuleInstanceDeleteModuleInstance(BaseModel):
    id: UUID


DeleteModuleInstance.model_rebuild()
DeleteModuleInstanceDeleteModuleInstance.model_rebuild()
