# Generated by ariadne-codegen
# Source: combined.graphql

from typing import List
from uuid import UUID

from .base_model import BaseModel


class Run(BaseModel):
    run: "RunRun"


class RunRun(BaseModel):
    id: UUID
    outs: List["RunRunOuts"]


class RunRunOuts(BaseModel):
    id: UUID


Run.model_rebuild()
RunRun.model_rebuild()
RunRunOuts.model_rebuild()
